@@ -41,6 +41,8 @@ from scipy import stats
 from scipy import special
 from scipy import ndimage
 
+from scipy.special import gamma
+
 from io import open
 
 ROOT2 = math.sqrt(2)
@@ -126,6 +128,11 @@ class Interpolator(object):
         return y
 
 
+# When we plot Hist, Pmf and Cdf objects, they don't appear in
+# the legend unless we override the default label.
+DEFAULT_LABEL = '_nolegend_' 
+
+
 class _DictWrapper(object):
     """An object that contains a dictionary."""
 
@@ -135,7 +142,7 @@ class _DictWrapper(object):
         obj: Hist, Pmf, Cdf, Pdf, dict, pandas Series, list of pairs
         label: string label
         """
-        self.label = label if label is not None else '_nolegend_'
+        self.label = label if label is not None else DEFAULT_LABEL
         self.d = {}
 
         # flag whether the distribution is under a log transform
@@ -165,12 +172,23 @@ class _DictWrapper(object):
 
     def __str__(self):
         cls = self.__class__.__name__
-        return '%s(%s)' % (cls, str(self.d))
+        if self.label == DEFAULT_LABEL:
+            return '%s(%s)' % (cls, str(self.d))
+        else:
+            return self.label
 
-    __repr__ = __str__
+    def __repr__(self):
+        cls = self.__class__.__name__
+        if self.label == DEFAULT_LABEL:
+            return '%s(%s)' % (cls, repr(self.d))
+        else:
+            return '%s(%s, %s)' % (cls, repr(self.d), repr(self.label))
 
     def __eq__(self, other):
-        return self.d == other.d
+        try:
+            return self.d == other.d
+        except AttributeError:
+            return False
 
     def __len__(self):
         return len(self.d)
@@ -281,6 +299,26 @@ class _DictWrapper(object):
         """Gets an unsorted sequence of (value, freq/prob) pairs."""
         return self.d.items()
 
+    def SortedItems(self):
+        """Gets a sorted sequence of (value, freq/prob) pairs.
+
+        It items are unsortable, the result is unsorted.
+        """
+        def isnan(x):
+            try:
+                return math.isnan(x)
+            except TypeError:
+                return False
+
+        if any([isnan(x) for x in self.Values()]):
+            msg = 'Keys contain NaN, may not sort correctly.'
+            logging.warning(msg)
+
+        try:
+            return sorted(self.d.items())
+        except TypeError:
+            return self.d.items()
+
     def Render(self, **options):
         """Generates a sequence of points suitable for plotting.
 
@@ -289,10 +327,7 @@ class _DictWrapper(object):
         Returns:
             tuple of (sorted value sequence, freq/prob sequence)
         """
-        if min(self.d.keys()) is np.nan:
-            logging.warning('Hist: contains NaN, may not render correctly.')
-
-        return zip(*sorted(self.Items()))
+        return zip(*self.SortedItems())
 
     def MakeCdf(self, label=None):
         """Makes a Cdf."""
@@ -301,7 +336,7 @@ class _DictWrapper(object):
 
     def Print(self):
         """Prints the values and freqs/probs in ascending order."""
-        for val, prob in sorted(self.d.items()):
+        for val, prob in self.SortedItems():
             print(val, prob)
 
     def Set(self, x, y=0):
@@ -432,7 +467,7 @@ class Pmf(_DictWrapper):
 
         returns: value from the Pmf
         """
-        p = percentage / 100.0
+        p = percentage / 100
         total = 0
         for val, prob in sorted(self.Items()):
             total += prob
@@ -465,43 +500,23 @@ class Pmf(_DictWrapper):
             t = [prob for (val, prob) in self.d.items() if val < x]
             return sum(t)
 
-    def __lt__(self, obj):
-        """Less than.
+    def ProbEqual(self, x):
+        """Probability that a sample from this Pmf is exactly x.
 
-        obj: number or _DictWrapper
+        x: number
 
         returns: float probability
         """
-        return self.ProbLess(obj)
+        if isinstance(x, _DictWrapper):
+            return PmfProbEqual(self, x)
+        else:
+            return self[x]
 
-    def __gt__(self, obj):
-        """Greater than.
+    # NOTE: I've decided to remove the magic comparators because they
+    # have the side-effect of making Pmf sortable, but in fact they
+    # don't support sorting.
 
-        obj: number or _DictWrapper
-
-        returns: float probability
-        """
-        return self.ProbGreater(obj)
-
-    def __ge__(self, obj):
-        """Greater than or equal.
-
-        obj: number or _DictWrapper
-
-        returns: float probability
-        """
-        return 1 - (self < obj)
-
-    def __le__(self, obj):
-        """Less than or equal.
-
-        obj: number or _DictWrapper
-
-        returns: float probability
-        """
-        return 1 - (self > obj)
-
-    def Normalize(self, fraction=1.0):
+    def Normalize(self, fraction=1):
         """Normalizes this PMF so the sum of all probs is fraction.
 
         Args:
@@ -513,10 +528,8 @@ class Pmf(_DictWrapper):
             raise ValueError("Normalize: Pmf is under a log transform")
 
         total = self.Total()
-        if total == 0.0:
+        if total == 0:
             raise ValueError('Normalize: total probability is zero.')
-            #logging.warning('Normalize: total probability is zero.')
-            #return total
 
         factor = fraction / total
         for x in self.d:
@@ -534,7 +547,7 @@ class Pmf(_DictWrapper):
             float value from the Pmf
         """
         target = random.random()
-        total = 0.0
+        total = 0
         for x, p in self.d.items():
             total += p
             if total >= target:
@@ -543,16 +556,29 @@ class Pmf(_DictWrapper):
         # we shouldn't get here
         raise ValueError('Random: Pmf might not be normalized.')
 
+    def Sample(self, n):
+        """Generates a random sample from this distribution.
+        
+        n: int length of the sample
+        returns: NumPy array
+        """
+        return self.MakeCdf().Sample(n)
+
     def Mean(self):
         """Computes the mean of a PMF.
 
         Returns:
             float mean
         """
-        mean = 0.0
-        for x, p in self.d.items():
-            mean += p * x
-        return mean
+        return sum(p * x for x, p in self.Items())
+
+    def Median(self):
+        """Computes the median of a PMF.
+
+        Returns:
+            float median
+        """
+        return self.MakeCdf().Percentile(50)
 
     def Var(self, mu=None):
         """Computes the variance of a PMF.
@@ -565,10 +591,15 @@ class Pmf(_DictWrapper):
         if mu is None:
             mu = self.Mean()
 
-        var = 0.0
-        for x, p in self.d.items():
-            var += p * (x - mu) ** 2
-        return var
+        return sum(p * (x-mu)**2 for x, p in self.Items())
+
+    def Expect(self, func):
+        """Computes the expectation of func(x).
+
+        Returns:
+            expectation
+        """
+        return np.sum(p * func(x) for x, p in self.Items()) 
 
     def Std(self, mu=None):
         """Computes the standard deviation of a PMF.
@@ -581,7 +612,7 @@ class Pmf(_DictWrapper):
         var = self.Var(mu)
         return math.sqrt(var)
 
-    def MaximumLikelihood(self):
+    def Mode(self):
         """Returns the value with the highest probability.
 
         Returns: float probability
@@ -589,6 +620,13 @@ class Pmf(_DictWrapper):
         _, val = max((prob, val) for val, prob in self.Items())
         return val
 
+    # The mode of a posterior is the maximum aposteori probability (MAP)
+    MAP = Mode
+
+    # If the distribution contains likelihoods only, the peak is the
+    # maximum likelihood estimator.
+    MaximumLikelihood = Mode
+
     def CredibleInterval(self, percentage=90):
         """Computes the central credible interval.
 
@@ -615,6 +653,8 @@ class Pmf(_DictWrapper):
         except AttributeError:
             return self.AddConstant(other)
 
+    __radd__ = __add__
+
     def AddPmf(self, other):
         """Computes the Pmf of the sum of values drawn from self and other.
 
@@ -625,7 +665,7 @@ class Pmf(_DictWrapper):
         pmf = Pmf()
         for v1, p1 in self.Items():
             for v2, p2 in other.Items():
-                pmf.Incr(v1 + v2, p1 * p2)
+                pmf[v1 + v2] += p1 * p2
         return pmf
 
     def AddConstant(self, other):
@@ -635,6 +675,9 @@ class Pmf(_DictWrapper):
 
         returns: new Pmf
         """
+        if other == 0:
+            return self.Copy()
+
         pmf = Pmf()
         for v1, p1 in self.Items():
             pmf.Set(v1 + other, p1)
@@ -737,7 +780,8 @@ class Pmf(_DictWrapper):
         returns: new Cdf
         """
         cdf = self.MakeCdf()
-        return cdf.Max(k)
+        cdf.ps **= k
+        return cdf
 
 
 class Joint(Pmf):
@@ -797,7 +841,7 @@ class Joint(Pmf):
         for prob, val in t:
             interval.append(val)
             total += prob
-            if total >= percentage / 100.0:
+            if total >= percentage / 100:
                 break
 
         return interval
@@ -915,7 +959,7 @@ def MakeMixture(metapmf, label='mix'):
     mix = Pmf(label=label)
     for pmf, p1 in metapmf.Items():
         for x, p2 in pmf.Items():
-            mix.Incr(x, p1 * p2)
+            mix[x] += p1 * p2
     return mix
 
 
@@ -933,7 +977,7 @@ def MakeUniformPmf(low, high, n):
     return pmf
 
 
-class Cdf(object):
+class Cdf:
     """Represents a cumulative distribution function.
 
     Attributes:
@@ -950,7 +994,7 @@ class Cdf(object):
         ps: list of cumulative probabilities
         label: string label
         """
-        self.label = label if label is not None else '_nolegend_'
+        self.label = label if label is not None else DEFAULT_LABEL
 
         if isinstance(obj, (_DictWrapper, Cdf, Pdf)):
             if not label:
@@ -995,9 +1039,19 @@ class Cdf(object):
         self.ps /= self.ps[-1]
 
     def __str__(self):
-        return 'Cdf(%s, %s)' % (str(self.xs), str(self.ps))
+        cls = self.__class__.__name__
+        if self.label == DEFAULT_LABEL:
+            return '%s(%s, %s)' % (cls, str(self.xs), str(self.ps))
+        else:
+            return self.label
 
-    __repr__ = __str__
+    def __repr__(self):
+        cls = self.__class__.__name__
+        if self.label == DEFAULT_LABEL:
+            return '%s(%s, %s)' % (cls, str(self.xs), str(self.ps))
+        else:
+            return '%s(%s, %s, %s)' % (cls, str(self.xs), str(self.ps), 
+                                       repr(self.label))
 
     def __len__(self):
         return len(self.xs)
@@ -1014,6 +1068,11 @@ class Cdf(object):
     def __eq__(self, other):
         return np.all(self.xs == other.xs) and np.all(self.ps == other.ps)
 
+    def Print(self):
+        """Prints the values and freqs/probs in ascending order."""
+        for val, prob in zip(self.xs, self.ps):
+            print(val, prob)
+
     def Copy(self, label=None):
         """Returns a copy of this Cdf.
 
@@ -1029,11 +1088,6 @@ class Cdf(object):
             label = self.label
         return Pmf(self, label=label)
 
-    def Values(self):
-        """Returns a sorted list of values.
-        """
-        return self.xs
-
     def Items(self):
         """Returns a sorted sequence of (value, probability) pairs.
 
@@ -1074,7 +1128,7 @@ class Cdf(object):
             float probability
         """
         if x < self.xs[0]:
-            return 0.0
+            return 0
         index = bisect.bisect(self.xs, x)
         p = self.ps[index-1]
         return p
@@ -1089,7 +1143,7 @@ class Cdf(object):
         xs = np.asarray(xs)
         index = np.searchsorted(self.xs, xs, side='right')
         ps = self.ps[index-1]
-        ps[xs < self.xs[0]] = 0.0
+        ps[xs < self.xs[0]] = 0
         return ps
 
     ProbArray = Probs
@@ -1109,15 +1163,20 @@ class Cdf(object):
         index = bisect.bisect_left(self.ps, p)
         return self.xs[index]
 
-    def ValueArray(self, ps):
+    def Values(self, ps=None):
         """Returns InverseCDF(p), the value that corresponds to probability p.
 
+        If ps is not provided, returns all values.
+
         Args:
             ps: NumPy array of numbers in the range [0, 1]
 
         Returns:
             NumPy array of values
         """
+        if ps is None:
+            return self.xs
+
         ps = np.asarray(ps)
         if np.any(ps < 0) or np.any(ps > 1):
             raise ValueError('Probability p must be in range [0, 1]')
@@ -1125,6 +1184,8 @@ class Cdf(object):
         index = np.searchsorted(self.ps, ps, side='left')
         return self.xs[index]
 
+    ValueArray = Values
+
     def Percentile(self, p):
         """Returns the value that corresponds to percentile p.
 
@@ -1134,7 +1195,19 @@ class Cdf(object):
         Returns:
             number value
         """
-        return self.Value(p / 100.0)
+        return self.Value(p / 100)
+
+    def Percentiles(self, ps):
+        """Returns the value that corresponds to percentiles ps.
+
+        Args:
+            ps: numbers in the range [0, 100]
+
+        Returns:
+            array of values
+        """
+        ps = np.asarray(ps)
+        return self.Values(ps / 100)
 
     def PercentileRank(self, x):
         """Returns the percentile rank of the value x.
@@ -1143,7 +1216,16 @@ class Cdf(object):
 
         returns: percentile rank in the range 0 to 100
         """
-        return self.Prob(x) * 100.0
+        return self.Prob(x) * 100
+
+    def PercentileRanks(self, xs):
+        """Returns the percentile ranks of the values in xs.
+
+        xs: potential value in the CDF
+
+        returns: array of percentile ranks in the range 0 to 100
+        """
+        return self.Probs(x) * 100
 
     def Random(self):
         """Chooses a random value from this distribution."""
@@ -1165,7 +1247,7 @@ class Cdf(object):
             float mean
         """
         old_p = 0
-        total = 0.0
+        total = 0
         for x, new_p in zip(self.xs, self.ps):
             p = new_p - old_p
             total += p * x
@@ -1183,13 +1265,13 @@ class Cdf(object):
         Returns:
             sequence of two floats, low and high
         """
-        prob = (1 - percentage / 100.0) / 2
+        prob = (1 - percentage / 100) / 2
         interval = self.Value(prob), self.Value(1 - prob)
         return interval
 
     ConfidenceInterval = CredibleInterval
 
-    def _Round(self, multiplier=1000.0):
+    def _Round(self, multiplier=1000):
         """
         An entry is added to the cdf only if the percentile differs
         from the previous value in a significant digit, where the number
@@ -1648,7 +1730,7 @@ def CredibleInterval(pmf, percentage=90):
         sequence of two floats, low and high
     """
     cdf = pmf.MakeCdf()
-    prob = (1 - percentage / 100.0) / 2
+    prob = (1 - percentage / 100) / 2
     interval = cdf.Value(prob), cdf.Value(1 - prob)
     return interval
 
@@ -1663,7 +1745,7 @@ def PmfProbLess(pmf1, pmf2):
     Returns:
         float probability
     """
-    total = 0.0
+    total = 0
     for v1, p1 in pmf1.Items():
         for v2, p2 in pmf2.Items():
             if v1 < v2:
@@ -1681,7 +1763,7 @@ def PmfProbGreater(pmf1, pmf2):
     Returns:
         float probability
     """
-    total = 0.0
+    total = 0
     for v1, p1 in pmf1.Items():
         for v2, p2 in pmf2.Items():
             if v1 > v2:
@@ -1699,7 +1781,7 @@ def PmfProbEqual(pmf1, pmf2):
     Returns:
         float probability
     """
-    total = 0.0
+    total = 0
     for v1, p1 in pmf1.Items():
         for v2, p2 in pmf2.Items():
             if v1 == v2:
@@ -1781,6 +1863,61 @@ def MakeBinomialPmf(n, p):
         pmf[k] = stats.binom.pmf(k, n, p)
     return pmf
 
+
+def EvalGammaPdf(x, a):
+    """Computes the Gamma PDF.
+
+    x: where to evaluate the PDF
+    a: parameter of the gamma distribution
+
+    returns: float probability
+    """
+    return x**(a-1) * np.exp(-x) / gamma(a)
+
+
+def MakeGammaPmf(xs, a):
+    """Makes a PMF discrete approx to a Gamma distribution.
+
+    lam: parameter lambda in events per unit time
+    xs: upper bound of the Pmf
+
+    returns: normalized Pmf
+    """
+    xs = np.asarray(xs)
+    ps = EvalGammaPdf(xs, a)
+    pmf = Pmf(dict(zip(xs, ps)))
+    pmf.Normalize()
+    return pmf
+
+
+def EvalGeometricPmf(k, p, loc=0):
+    """Evaluates the geometric PMF.
+
+    With loc=0: Probability of `k` trials to get one success.
+    With loc=-1: Probability of `k` trials before first success.
+
+    k: number of trials
+    p: probability of success on each trial
+    """
+    return stats.geom.pmf(k, p, loc=loc)
+    
+
+def MakeGeometricPmf(p, loc=0, high=10):
+    """Evaluates the binomial PMF.
+
+    With loc=0: PMF of trials to get one success.
+    With loc=-1: PMF of trials before first success.
+
+    p: probability of success
+    high: upper bound where PMF is truncated
+    """
+    pmf = Pmf()
+    for k in range(high):
+        pmf[k] = stats.geom.pmf(k, p, loc=loc)
+    pmf.Normalize()
+    return pmf
+
+
 def EvalHypergeomPmf(k, N, K, n):
     """Evaluates the hypergeometric PMF.
 
@@ -1798,10 +1935,7 @@ def EvalPoissonPmf(k, lam):
 
     returns: float probability
     """
-    # don't use the scipy function (yet).  for lam=0 it returns NaN;
-    # should be 0.0
-    # return stats.poisson.pmf(k, lam)
-    return lam ** k * math.exp(-lam) / special.gamma(k+1)
+    return stats.poisson.pmf(k, lam)
 
 
 def MakePoissonPmf(lam, high, step=1):
@@ -1814,7 +1948,7 @@ def MakePoissonPmf(lam, high, step=1):
     """
     pmf = Pmf()
     for k in range(0, high + 1, step):
-        p = EvalPoissonPmf(k, lam)
+        p = stats.poisson.pmf(k, lam)
         pmf.Set(k, p)
     pmf.Normalize()
     return pmf
@@ -1853,6 +1987,67 @@ def MakeExponentialPmf(lam, high, n=200):
     return pmf
 
 
+def EvalWeibullPdf(x, lam, k):
+    """Computes the Weibull PDF.
+
+    x: value
+    lam: parameter lambda in events per unit time
+    k: parameter
+
+    returns: float probability density
+    """
+    arg = (x / lam)
+    return k / lam * arg**(k-1) * np.exp(-arg**k)
+
+
+def EvalWeibullCdf(x, lam, k):
+    """Evaluates CDF of the Weibull distribution."""
+    arg = (x / lam)
+    return 1 - np.exp(-arg**k)
+
+
+def MakeWeibullPmf(lam, k, high, n=200):
+    """Makes a PMF discrete approx to a Weibull distribution.
+
+    lam: parameter lambda in events per unit time
+    k: parameter
+    high: upper bound
+    n: number of values in the Pmf
+
+    returns: normalized Pmf
+    """
+    xs = np.linspace(0, high, n)
+    ps = EvalWeibullPdf(xs, lam, k)
+    ps[np.isinf(ps)] = 0
+    return Pmf(dict(zip(xs, ps)))
+
+
+def EvalParetoPdf(x, xm, alpha):
+    """Computes the Pareto.
+
+    xm: minimum value (scale parameter)
+    alpha: shape parameter
+
+    returns: float probability density
+    """
+    return stats.pareto.pdf(x, alpha, scale=xm)
+
+
+def MakeParetoPmf(xm, alpha, high, num=101):
+    """Makes a PMF discrete approx to a Pareto distribution.
+
+    xm: minimum value (scale parameter)
+    alpha: shape parameter
+    high: upper bound value
+    num: number of values
+
+    returns: normalized Pmf
+    """
+    xs = np.linspace(xm, high, num)
+    ps = stats.pareto.pdf(xs, alpha, scale=xm)
+    pmf = Pmf(dict(zip(xs, ps)))
+    return pmf
+
 def StandardNormalCdf(x):
     """Evaluates the CDF of the standard Normal distribution.
     
@@ -1965,7 +2160,7 @@ def RenderParetoCdf(xmin, alpha, low, high, n=50):
     return xs, ps
 
 
-class Beta(object):
+class Beta:
     """Represents a Beta distribution.
 
     See http://en.wikipedia.org/wiki/Beta_distribution
@@ -1989,6 +2184,12 @@ class Beta(object):
         """Computes the mean of this distribution."""
         return self.alpha / (self.alpha + self.beta)
 
+    def MAP(self):
+        """Computes the value with maximum a posteori probability."""
+        a = self.alpha - 1
+        b = self.beta - 1
+        return a / (a + b)
+
     def Random(self):
         """Generates a random variate from this distribution."""
         return random.betavariate(self.alpha, self.beta)
@@ -2022,6 +2223,9 @@ class Beta(object):
         model of the continuous distribution, and behaves well as
         the number of values increases.
         """
+        if label is None and self.label is not None:
+            label = self.label
+
         if self.alpha < 1 or self.beta < 1:
             cdf = self.MakeCdf()
             pmf = cdf.MakePmf()
@@ -2168,7 +2372,7 @@ def LogBinomialCoef(n, k):
     return n * math.log(n) - k * math.log(k) - (n - k) * math.log(n - k)
 
 
-def NormalProbability(ys, jitter=0.0):
+def NormalProbability(ys, jitter=0):
     """Generates data for a normal probability plot.
 
     ys: sequence of values
@@ -2637,12 +2841,15 @@ def ReadStataDct(dct_file, **options):
 
     returns: FixedWidthVariables object
     """
-    type_map = dict(byte=int, int=int, long=int, float=float, double=float)
+    type_map = dict(byte=int, int=int, long=int, float=float, 
+                    double=float, numeric=float)
 
     var_info = []
-    for line in open(dct_file, **options):
-        match = re.search( r'_column\(([^)]*)\)', line)
-        if match:
+    with open(dct_file, **options) as f:
+        for line in f:
+            match = re.search( r'_column\(([^)]*)\)', line)
+            if not match:
+                continue
             start = int(match.group(1))
             t = line.split()
             vtype, name, fstring = t[1:4]
@@ -2710,9 +2917,9 @@ def ResampleRowsWeighted(df, column='finalwgt'):
 
     returns: DataFrame
     """
-    weights = df[column]
-    cdf = Cdf(dict(weights))
-    indices = cdf.Sample(len(weights))
+    weights = df[column].copy()
+    weights /= sum(weights)
+    indices = np.random.choice(df.index, len(df), replace=True, p=weights)
     sample = df.loc[indices]
     return sample
 
